#include "formats.h"

enum {
	XMPPTXT_MODULE_NAME,

	XMPPTXT_FILL_1,

	XMPPTXT_FORMAT_NAME,
	XMPPTXT_FORMAT_JID,
	XMPPTXT_FORMAT_RESOURCE,
	XMPPTXT_FORMAT_RESOURCE_SHOW,
	XMPPTXT_FORMAT_RESOURCE_STATUS,
	XMPPTXT_FORMAT_SUBSCRIPTION,

	XMPPTXT_FILL_2,

	XMPPTXT_ROSTER_GROUP,
	XMPPTXT_ROSTER_CONTACT,
	XMPPTXT_BEGIN_OF_ROSTER,
	XMPPTXT_END_OF_ROSTER,
	XMPPTXT_NOT_IN_ROSTER,

	XMPPTXT_FILL_3,

	XMPPTXT_SUBSCRIBE,
	XMPPTXT_SUBSCRIBED,
	XMPPTXT_UNSUBSCRIBE,
	XMPPTXT_UNSUBSCRIBED,

	XMPPTXT_FILL_4,

	XMPPTXT_MESSAGE_EVENT,
	XMPPTXT_MESSAGE_NOT_DELIVERED,
	XMPPTXT_MESSAGE_TIMESTAMP,

	XMPPTXT_FILL_5,

	XMPPTXT_QUERY_AKA,

	XMPPTXT_FILL_6,

	XMPPTXT_CHANNEL_JOINERROR,

	XMPPTXT_FILL_7,

	XMPPTXT_PRESENCE_CHANGE,
	XMPPTXT_PRESENCE_CHANGE_REASON,

	XMPPTXT_FILL_8,

	XMPPTXT_VCARD,
	XMPPTXT_VCARD_VALUE,
	XMPPTXT_VCARD_SUBVALUE,
	XMPPTXT_END_OF_VCARD,

	XMPPTXT_FILL_9,

	XMPPTXT_RAW_IN_HEADER,
	XMPPTXT_RAW_OUT_HEADER,
	XMPPTXT_RAW_MESSAGE,
	XMPPTXT_DEFAULT_EVENT,
	XMPPTXT_DEFAULT_ERROR,

	XMPPTXT_FILL_10,

	XMPPTXT_REGISTRATION_STARTED,
	XMPPTXT_REGISTRATION_SUCCEED,
	XMPPTXT_REGISTRATION_FAILED,
};

extern FORMAT_REC fecommon_xmpp_formats[];
